"""
Game
"""

import pygame, Numeric, pickle, math, pygame.font
from pygame.locals import *

import tile_engine, sprite_repository
import sprites.player, sprites.thug, sprites.cherub

import psyco
#psyco.log()
psyco.profile()

class Game:
    def __init__(self):
        self.init_display()
        self.init_map()
        self.init_sprites()
        self.init_sound()
        
        self.frame_count = 0
        self.view_pos = [620, 312]
        self.scroll_frame = Rect(80, 50, 320 - 80 * 2, 200 - 50 * 2)
        self.full_draw_flag = 1

        self.clock = pygame.time.Clock()

    def init_display(self):
        pygame.init()

        self.screen = pygame.display.set_mode((320, 240), HWSURFACE | DOUBLEBUF, 32)
        self.view = self.screen.subsurface(Rect(0, 0, 320, 200))
        self.interface = self.screen.subsurface(Rect(0, 200, 320, 40))
        self.background = pygame.image.load("data/background.jpg")
        self.background = self.background.convert()

        self.game_font = pygame.font.Font("data/font.ttf", 10)
        
    def init_map(self):
        f = file("data/test.map", "r")
        self.t = tile_engine.tile_engine(pickle.load(f), "data/tiles")
        f.close()

        self.t.make_minimap()

    def init_sprites(self):
        """Initialize the sprite repository, then load all the initial sprites."""
        self.sr = sprite_repository.sprite_repository("data/sprites")

        self.player = sprites.player.player(self, [16, 446], 0, "Stand")
        self.sl = [self.player, sprites.cherub.cherub(self, [800, 380], 0, "Float")]
        
    def init_sound(self):
        pygame.mixer.init()
        pygame.mixer.music.load('data/song.ogg')
        pygame.mixer.music.play(-1)
            
    def draw(self):
        self.draw_environment()
        self.draw_sprites()
        self.draw_interface()
    
        pygame.display.flip()

    def draw_environment(self):
        if self.full_draw_flag:
            self.draw_background()
        self.t.draw_map(self.view, self.view_pos, self.full_draw_flag)
        
#        self.full_draw_flag = 1

    def draw_background(self):
        # Nice parallax blit
        x = float(self.view_pos[0]) / float((self.t.map_tile_width * self.t.tile_size) - 320) * (self.background.get_width() - 320)
        y = float(self.view_pos[1]) / float((self.t.map_tile_height * self.t.tile_size) - 200) * (self.background.get_height() - 200)
        self.view.blit(self.background, (0, 0), Rect(x, y, 320, 200))

    def draw_interface(self):
        self.draw_minimap()
        
    def draw_minimap(self):
#        self.interface.blit(self.t.minimap, (0, 0), (0, 0, min(self.t.minimap.get_width(), 48), min(self.t.minimap.get_height(), 32)))
        self.interface.blit(self.t.minimap, (0, 0))

        x, y = self.player.r.x // self.t.tile_size, self.player.r.y // self.t.tile_size
        w, h = math.ceil(float(self.player.r.w) / float(self.t.tile_size)), math.ceil(float(self.player.r.h) / float(self.t.tile_size))
        self.interface.fill((255, 255, 255), (x, y, w, h))
        
    def draw_sprites(self):
        for s in self.sl:
            s.draw(self.view)
#        self.player.draw(self.view)

    def view_transform(self, r, o=(0,0)):
        return Rect(r.x - self.view_pos[0] + o[0], r.y - self.view_pos[1] + o[1], r.w, r.h)

    def handle_events(self):
        e = pygame.event.poll()
        while e.type != NOEVENT:
            if e.type == QUIT:
                self.running = 0
            elif e.type == KEYDOWN:
                self.handle_keydown_event(e.key)
            e = pygame.event.poll()

    def handle_keydown_event(self, k):
        if k == K_ESCAPE:
            self.running = 0

    def handle_mouse_state(self):
        pass

    def update_sprites(self):
#        self.player.update()
        for s in self.sl:
            s.update()
            
    def update(self):
        # Update the sprites first
        self.update_sprites()

        # Scrolling
        s = self.scroll_frame
        if self.player.r.x < self.view_pos[0] + s.x:
            self.view_pos[0] = max(self.player.r.x - s.x, 0)
        if self.player.r.y < self.view_pos[1] + s.y:
            self.view_pos[1] = max(self.player.r.y - s.y, 0)
        if self.player.r.x + self.player.r.w > self.view_pos[0] + s.x + s.w:
            self.view_pos[0] = min((self.player.r.x + self.player.r.w) - (s.x + s.w), len(self.t.map[0]) * self.t.tile_size - 320)        
        if self.player.r.y + self.player.r.h > self.view_pos[1] + s.y + s.h:
            self.view_pos[1] = min((self.player.r.y + self.player.r.h) - (s.y + s.h), len(self.t.map) * self.t.tile_size - 200)        
            
    def main(self):        
        self.running = 1
        while self.running:
            self.frame_count += 1
            self.clock.tick(80)
            
            self.handle_mouse_state()
            self.handle_events()

            self.t.update()
            self.update()

            self.draw()


        pygame.quit()

# Execute main function
if __name__ == "__main__":
    game = Game()
    game.main()
